# This file was auto-generated by Fern from our API Definition.

from .environment import GroTradeFlowEnvironment
from .resources.trade_flow.client import AsyncTradeFlowClient, TradeFlowClient


class GroTradeFlow:
    def __init__(self, *, environment: GroTradeFlowEnvironment = GroTradeFlowEnvironment.PRODUCTION):
        self._environment = environment
        self.trade_flow = TradeFlowClient(environment=self._environment)


class AsyncGroTradeFlow:
    def __init__(self, *, environment: GroTradeFlowEnvironment = GroTradeFlowEnvironment.PRODUCTION):
        self._environment = environment
        self.trade_flow = AsyncTradeFlowClient(environment=self._environment)
